
library(mlogit);    library(glmnet);    library(ggplot2)

set.seed(1234)

# load functions 

setwd('/Code_Paper/Functions')

source("halton_draws.R");       source("cdfEst.R");       source("estimator.R");      source("crossValidation.R")


# load data from mlogit package

data("ModeCanada")




# prepare data

# remove bus as an alternative

bususers = with(ModeCanada, case[choice == 1 & alt == "bus"])
trainusers = with(ModeCanada, case[choice == 1 & alt == "train"])
plainuser = with(ModeCanada, case[choice == 1 & alt == "air"])
carusers = with(ModeCanada, case[choice == 1 & alt == "car"])

ModeCanada$choiceNum[ ModeCanada$case %in% bususers] = 4
ModeCanada$choiceNum[ ModeCanada$case %in% plainuser] = 2
ModeCanada$choiceNum[ ModeCanada$case %in% carusers] = 3
ModeCanada$choiceNum[ ModeCanada$case %in% trainusers] = 1

busAlt = with(ModeCanada, case[alt == "bus"])
ModeCanada$noalt = ifelse(ModeCanada$case %in% busAlt, ModeCanada$noalt - 1, ModeCanada$noalt )
ModeCanada = subset(ModeCanada, ! case %in% bususers)
ModeCanada = subset(ModeCanada, alt != "bus")
ModeCanada$alt = ModeCanada$alt[drop = TRUE]
ModeCanada = subset(ModeCanada, noalt == 3)

# create variable travel time as sum of in- and out-of-vehicle travel time

ModeCanada$tt = ModeCanada$ivt + ModeCanada$ovt

# create urban dummy

ModeCanada$urbanDummy = ifelse(ModeCanada$urban == 0, 0, 1)

# create dummy for every alternative 

ModeCanada$IntTrain = ifelse(ModeCanada$alt == "train", 1, 0)
ModeCanada$IntAir = ifelse(ModeCanada$alt == "air", 1, 0)
ModeCanada$IntBus = ifelse(ModeCanada$alt == "bus", 1, 0)

# interact income with alternative dummies

ModeCanada$IncomeTrain = ifelse(ModeCanada$alt == "train", ModeCanada$income, 0)
ModeCanada$IncomeAir = ifelse(ModeCanada$alt == "air", ModeCanada$income, 0)
ModeCanada$IncomeBus = ifelse(ModeCanada$alt == "bus", ModeCanada$income, 0)

# interacr distance with alternative dummies

ModeCanada$DistTrain = ifelse(ModeCanada$alt == "train", ModeCanada$dist, 0)
ModeCanada$DistAir = ifelse(ModeCanada$alt == "air", ModeCanada$dist, 0)
ModeCanada$DistBus = ifelse(ModeCanada$alt == "bus", ModeCanada$dist, 0)

# interact urban dummy with alternative dummies

ModeCanada$UrbanDTrain = ifelse(ModeCanada$alt == "train", ModeCanada$urbanDummy, 0)
ModeCanada$UrbanDAir = ifelse(ModeCanada$alt == "air", ModeCanada$urbanDummy, 0)
ModeCanada$UrbanDBus = ifelse(ModeCanada$alt == "bus", ModeCanada$urbanDummy, 0)




# estimate model with semiparametric estimator (assume travel time is normally distributed)

ModeCanadaMlogit = mlogit.data(ModeCanada, shape = 'long', chid.var = 'case', alt.var = 'alt', choice = 'choice', drop.index = F)

out = mlogit(choice ~ -1 + IntAir + IntTrain + freq + cost + tt + IncomeTrain + IncomeAir + DistTrain + DistAir + UrbanDTrain + UrbanDAir, 
               ModeCanadaMlogit, rpar = c( tt ='n'), correlation = FALSE, halton = NA, R = 100, tol = 1e-10, reflevel = "car")
              
             






# estimate with nonparametric estimator

# create matrix and vector for fixed and random coefficients

xdataFix = as.matrix(ModeCanada[, c("IntAir", "IntTrain", "freq", "cost", "IncomeTrain", "IncomeAir", "DistTrain", 
                                     "DistAir", "UrbanDTrain", "UrbanDAir")]) 

xdataRand = as.matrix(ModeCanada$tt)

dependentData = as.vector(ModeCanada$choice)


# define parameter for estimation with FKRB and generalized estimator 

n = length(unique(ModeCanada$case));      R = 100;      ind = ModeCanada$case;      j = 3


# use fixed coefficients for estimation and use random for the definition of the fixed grid 

betaFix = coefficients(out)[c("IntAir", "IntTrain", "freq", "cost", "IncomeTrain", "IncomeAir", "DistTrain", "DistAir", "UrbanDTrain", "UrbanDAir")]

# define uniform grid

gridR = seq(from = coefficients(out)["tt"] - 3 * coefficients(out)["sd.tt"], to = coefficients(out)["tt"] + 3 * coefficients(out)["sd.tt"], length.out = R)

# calculate choice probabilities at every grid point

linearIndexFixed = xdataFix %*% betaFix

linearIndexRand = matrix(xdataRand, nrow = length(xdataRand), ncol = 1) %*% t(gridR)

linearIndex = matrix(rep(linearIndexFixed, times = R), nrow = length(linearIndexFixed), ncol = R) + linearIndexRand

expLinearIndex = exp(linearIndex)

expLinearIndexSum = (matrix(rep(diag(1, n), each = j), nrow = n, ncol = j * n, byrow = TRUE) %*% expLinearIndex)

choiceProbsR = expLinearIndex / expLinearIndexSum[rep(1:n, each = j), ]



# estimate model with FKRB estimator 

thetaFKRB = estimator(choiceProbsR, dependentData, 0, j, eps = 1e-05)$theta


# estimate with generalized estimator

mus = c(0, glmnet(choiceProbsR, dependentData, alpha = 0, intercept = FALSE, lower.limits = rep(0, R), standardize = FALSE)$lambda)

CvMus = crossValidation(choiceProbsR, dependentData, mus, 10, j)

thetaOneSe = estimator(choiceProbsR, dependentData, CvMus$muOneSe, j, eps = 1e-05)$theta


# calculate estimated CDFs

gridE = seq(from = coefficients(out)["tt"] - 3 * coefficients(out)["sd.tt"], to = coefficients(out)["tt"] + 3 * coefficients(out)["sd.tt"], length.out = 500)

CDFFKRB = cdfEst(matrix(gridE, nrow = length(gridE), ncol = 1), matrix(gridR, nrow = length(gridR), ncol = 1), thetaFKRB)

CDFOneSe = cdfEst(matrix(gridE, nrow = length(gridE), ncol = 1), matrix(gridR, nrow = length(gridR), ncol = 1), thetaOneSe)

CDFSemi = pnorm(gridE, mean = coefficients(out)["tt"], sd = coefficients(out)["sd.tt"])


# plot distributions

# FKRB estimator
ggplot(data = data.frame(x = gridR, theta = thetaFKRB)) + geom_point(aes(x = x, y = theta, color = I("red"))) + 
  scale_x_continuous(name = expression(beta)) + ylab(expression(theta^FKRB)) + 
  theme(panel.grid.major.x = element_blank(), panel.grid.major.y = element_line(size = 0.5, color = "white"),
        panel.background = element_rect(fill = "white", colour = "black")) 
# generalized estimator
ggplot(data = data.frame(x = gridR, theta = thetaOneSe)) + geom_point(aes(x = x, y = theta, color = I("blue"))) + 
  scale_x_continuous(name = expression(beta)) + ylab(expression(theta^ENet)) + #ylim(0, 0.125) + 
  theme(panel.grid.major.x = element_blank(), panel.grid.major.y = element_line(size = 0.5, color = "white"),
        panel.background = element_rect(fill = "white", colour = "black")) 
# semiparametric estimator
ggplot(data = data.frame(x = c(min(gridR), max(gridR))), aes(x)) + 
  stat_function(fun = dnorm, n = 1000, args = list(mean = coef(out)["tt"], sd = coef(out)["sd.tt"])) + ylab("Density") +
  scale_x_continuous(name = expression(beta)) + theme(panel.grid.major.x = element_blank(), panel.grid.major.y = element_line(size = 0.5, color = "white"),
        panel.background = element_rect(fill = "white", colour = "black")) 

# plot estimated CDFs

data = rbind(data.frame(beta = gridE, CDF = CDFSemi, Type = 'Semi'), data.frame(beta = gridE, CDF = CDFFKRB, Type = 'FKRB'), 
             data.frame(beta = gridE, CDF = CDFOneSe, Type = 'OneSe'))

ggplot(data = data) + geom_line(data=subset(data, Type == "Semi"), aes(x = beta, y = CDF, color = Type, group = Type), size = 1.05) + 
  geom_step(data=subset(data, Type != "Semi"), aes(x = beta, y = CDF, color=Type, group = Type), size = 1.05) + 
  ylab(expression(F(beta))) + xlab(expression(beta)) +
  theme(panel.grid.major.x = element_blank(), panel.grid.major.y = element_line(size = 0.5, color = "white"),
        panel.background = element_rect(fill = "white", colour = "black")) 





# calculate own and cross-travel time elasticities for nonparametric estimators

# specify probability weights and beta values


# elasticities for FKRB

theta = thetaFKRB   

betaR = matrix(rep(gridR, times = nrow(choiceProbsR)), nrow = nrow(choiceProbsR), ncol = R, byrow = TRUE)

# calculate unconditonal choice probabilities

choiceProbs = choiceProbsR %*% theta

# get index for every alternative

train = which(ModeCanada$alt == "train")
air = which(ModeCanada$alt == "air")
car = which(ModeCanada$alt == "car")

# calculate own-travel time elasticities 

ownElasticity = (xdataRand / choiceProbs) * (betaR * choiceProbsR * (1 - choiceProbsR)) %*% theta

# calculate cross-travel time elasticities 

# car
crossElasticityCarTrain = -(xdataRand[train] / choiceProbs[car]) * (betaR[car, ] * choiceProbsR[car, ] * choiceProbsR[train, ]) %*% theta
crossElasticityCarAir = -(xdataRand[air] / choiceProbs[car]) * (betaR[car, ] * choiceProbsR[car, ] * choiceProbsR[air, ]) %*% theta
# air
crossElasticityAirCar = -(xdataRand[car] / choiceProbs[air]) * (betaR[air, ] * choiceProbsR[air, ] * choiceProbsR[car, ]) %*% theta
crossElasticityAirTrain = -(xdataRand[train] / choiceProbs[air]) * (betaR[air, ] * choiceProbsR[air, ] * choiceProbsR[train, ]) %*% theta
# train
crossElasticityTrainCar = -(xdataRand[car] / choiceProbs[train]) * (betaR[train, ] * choiceProbsR[train, ] * choiceProbsR[car, ]) %*% theta
crossElasticityTrainAir = -(xdataRand[air] / choiceProbs[train]) * (betaR[train, ] * choiceProbsR[train, ] * choiceProbsR[air, ]) %*% theta

# save in table

ElasticityTableMeanFKRB = rbind(c(mean(ownElasticity[car]), mean(crossElasticityAirCar), mean(crossElasticityTrainCar)),
                        c(mean(crossElasticityCarAir), mean(ownElasticity[air]), mean(crossElasticityTrainAir)), 
                        c(mean(crossElasticityCarTrain), mean(crossElasticityAirTrain), mean(ownElasticity[train])))


ElasticityTableMedianFKRB = rbind(c(median(ownElasticity[car]), median(crossElasticityAirCar), median(crossElasticityTrainCar)),
                              c(median(crossElasticityCarAir), median(ownElasticity[air]), median(crossElasticityTrainAir)), 
                              c(median(crossElasticityCarTrain), median(crossElasticityAirTrain), median(ownElasticity[train])))


texFKRB <- matrix(ncol=3,nrow=3)
colnames(texFKRB) <- rownames(texFKRB) <- c("Car & ", "Air & ", "Train & ")
texFKRB[] <- paste0(round(ElasticityTableMeanFKRB,4)," \\", "scriptsize{ (",round(ElasticityTableMedianFKRB,4),")} &")




# change working directory to save files
setwd('../Application')


# write into tex file

sink("ElasticitiesFKRB.txt")
prmatrix(texFKRB, quote = FALSE)
sink()





# elasticities for OneSe

theta = thetaOneSe  

betaR = matrix(rep(gridR, times = nrow(choiceProbsR)), nrow = nrow(choiceProbsR), ncol = R, byrow = TRUE)

# calculate unconditonal choice probabilities

choiceProbs = choiceProbsR %*% theta

# get index for every alternative

train = which(ModeCanada$alt == "train")
air = which(ModeCanada$alt == "air")
car = which(ModeCanada$alt == "car")

# calculate own-travel time elasticities 

ownElasticity = (xdataRand / choiceProbs) * (betaR * choiceProbsR * (1 - choiceProbsR)) %*% theta

# calculate cross-travel time elasticities 

# car
crossElasticityCarTrain = -(xdataRand[train] / choiceProbs[car]) * (betaR[car, ] * choiceProbsR[car, ] * choiceProbsR[train, ]) %*% theta
crossElasticityCarAir = -(xdataRand[air] / choiceProbs[car]) * (betaR[car, ] * choiceProbsR[car, ] * choiceProbsR[air, ]) %*% theta
# air
crossElasticityAirCar = -(xdataRand[car] / choiceProbs[air]) * (betaR[air, ] * choiceProbsR[air, ] * choiceProbsR[car, ]) %*% theta
crossElasticityAirTrain = -(xdataRand[train] / choiceProbs[air]) * (betaR[air, ] * choiceProbsR[air, ] * choiceProbsR[train, ]) %*% theta
# train
crossElasticityTrainCar = -(xdataRand[car] / choiceProbs[train]) * (betaR[train, ] * choiceProbsR[train, ] * choiceProbsR[car, ]) %*% theta
crossElasticityTrainAir = -(xdataRand[air] / choiceProbs[train]) * (betaR[train, ] * choiceProbsR[train, ] * choiceProbsR[air, ]) %*% theta

# save in table

ElasticityTableMeanOneSe = rbind(c(mean(ownElasticity[car]), mean(crossElasticityAirCar), mean(crossElasticityTrainCar)),
                                c(mean(crossElasticityCarAir), mean(ownElasticity[air]), mean(crossElasticityTrainAir)), 
                                c(mean(crossElasticityCarTrain), mean(crossElasticityAirTrain), mean(ownElasticity[train])))


ElasticityTableMedianOneSe = rbind(c(median(ownElasticity[car]), median(crossElasticityAirCar), median(crossElasticityTrainCar)),
                                  c(median(crossElasticityCarAir), median(ownElasticity[air]), median(crossElasticityTrainAir)), 
                                  c(median(crossElasticityCarTrain), median(crossElasticityAirTrain), median(ownElasticity[train])))


texOneSe <- matrix(ncol=3,nrow=3)
colnames(texOneSe) <- rownames(texOneSe) <- c("Car & ", "Air & ", "Train & ")
texOneSe[] <- paste0(round(ElasticityTableMeanOneSe,4)," \\", "scriptsize{ (",round(ElasticityTableMedianOneSe,4),")} &")





# write into tex file

sink("ElasticitiesOneSe.txt")
prmatrix(texOneSe, quote = FALSE)
sink()



# calculate own and cross-travel time elasticities elasticities for semiparametric estimator

# specify probability weights and beta values
set.seed(1234)
betaR = seq(from = -0.2, to = 0.2, length.out = 1000)

theta =  dnorm(betaR, mean = coef(out)["tt"], sd = coef(out)["sd.tt"])
theta = theta / sum(theta)

# calculate choice probabilities

linearIndexFixed = xdataFix %*% betaFix

linearIndexRand = matrix(xdataRand, nrow = length(xdataRand), ncol = 1) %*% t(betaR)

linearIndex = matrix(rep(linearIndexFixed, times = length(betaR)), nrow = length(linearIndexFixed), ncol = length(betaR)) + linearIndexRand

expLinearIndex = exp(linearIndex)

expLinearIndexSum = (matrix(rep(diag(1, n), each = j), nrow = n, ncol = j * n, byrow = TRUE) %*% expLinearIndex)

choiceProbsNorm = expLinearIndex / expLinearIndexSum[rep(1:n, each = j), ]

# calculate unconditonal choice probabilities

choiceProbs = choiceProbsNorm %*% theta

# calculate own-travel time elasticities 

betaR = matrix(rep(betaR, times = nrow(choiceProbsNorm)), nrow = nrow(choiceProbsNorm), ncol = length(betaR), byrow = TRUE)

ownElasticity = (xdataRand / choiceProbs) * (betaR * choiceProbsNorm * (1 - choiceProbsNorm)) %*% theta


# calculate cross-travel time elasticities

#car
crossElasticityCarTrain = -(xdataRand[train] / choiceProbs[car]) * (betaR[car, ] * choiceProbsNorm[car, ] * choiceProbsNorm[train, ]) %*% theta
crossElasticityCarAir = -(xdataRand[air] / choiceProbs[car]) * (betaR[car, ] * choiceProbsNorm[car, ] * choiceProbsNorm[air, ]) %*% theta
# air
crossElasticityAirCar = -(xdataRand[car] / choiceProbs[air]) * (betaR[air, ] * choiceProbsNorm[air, ] * choiceProbsNorm[car, ]) %*% theta
crossElasticityAirTrain = -(xdataRand[train] / choiceProbs[air]) * (betaR[air, ] * choiceProbsNorm[air, ] * choiceProbsNorm[train, ]) %*% theta
# train
crossElasticityTrainCar = -(xdataRand[car] / choiceProbs[train]) * (betaR[train, ] * choiceProbsNorm[train, ] * choiceProbsNorm[car, ]) %*% theta
crossElasticityTrainAir = -(xdataRand[air] / choiceProbs[train]) * (betaR[train, ] * choiceProbsNorm[train, ] * choiceProbsNorm[air, ]) %*% theta

# save in table

ElasticityTableMeanSemi = rbind(c(mean(ownElasticity[car]), mean(crossElasticityAirCar), mean(crossElasticityTrainCar)),
                        c(mean(crossElasticityCarAir), mean(ownElasticity[air]), mean(crossElasticityTrainAir)), 
                        c(mean(crossElasticityCarTrain), mean(crossElasticityAirTrain), mean(ownElasticity[train])))

ElasticityTableMedianSemi = rbind(c(median(ownElasticity[car]), median(crossElasticityAirCar), median(crossElasticityTrainCar)),
                        c(median(crossElasticityCarAir), median(ownElasticity[air]), median(crossElasticityTrainAir)), 
                        c(median(crossElasticityCarTrain), median(crossElasticityAirTrain), median(ownElasticity[train])))


texSemi <- matrix(ncol=3,nrow=3)
colnames(texSemi) <- rownames(texSemi) <- c("Car & ", "Air & ", "Train & ")

texSemi[] <- paste0(round(ElasticityTableMeanSemi,4)," \\", "scriptsize{ (",round(ElasticityTableMedianSemi,4),")} &")


# write into text file

sink("ElasticitiesNorm.txt")
prmatrix(texSemi, quote = FALSE)

sink()





# calculate ratio of elasticities of FKRB divided by those of ENet
texRatioFKRB_ENet <- matrix(ncol=3,nrow=3)
colnames(texRatioFKRB_ENet) <- rownames(texRatioFKRB_ENet) <- c("Car & ", "Air & ", "Train & ")

texRatioFKRB_ENet[] <- paste0(round(ElasticityTableMeanFKRB/ElasticityTableMeanOneSe,4),
                              " \\", "scriptsize{ (",round(ElasticityTableMeanFKRB/ElasticityTableMedianOneSe,4),")} &")



# calculate ratio of elasticities of semiparametric divided by those of ENet
texRatioSemi_ENet <- matrix(ncol=3,nrow=3)
colnames(texRatioSemi_ENet) <- rownames(texRatioSemi_ENet) <- c("Car & ", "Air & ", "Train & ")

texRatioSemi_ENet[] <- paste0(round(ElasticityTableMeanSemi/ElasticityTableMeanOneSe,4),
                              " \\", "scriptsize{ (",round(ElasticityTableMedianSemi/ElasticityTableMedianOneSe,4),")} &")



sink("ElasticitiesRatio.txt")

cat("Elasticities of FKRB divided by those of ENet")
prmatrix(texRatioFKRB_ENet, quote = FALSE)

cat("\n\n")

cat("Elasticities of semiparametric divided by those of ENet")
prmatrix(texRatioSemi_ENet, quote = FALSE)

sink()





